@@ -4,57 +4,53 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import codecs
 import sys, os
 
-from os.path import abspath, basename, exists, isdir, isfile, join
+from os.path import isdir, isfile, join
 
 from conda.cli import common as cli_common
 from conda.cli.main import generate_parser, init_loggers
 from conda.base.context import context
-from conda.core.index import calculate_channel_urls, check_whitelist, _supplement_index_with_system #, get_index
-from conda.models.channel import Channel, prioritize_channels
-from conda.models.records import PackageRecord
 from conda.models.match_spec import MatchSpec
-from conda.cli.main_list import list_packages
 from conda.core.prefix_data import PrefixData
-from conda.misc import clone_env, explicit, touch_nonadmin
-from conda.common.serialize import json_dump
+from conda.misc import explicit, touch_nonadmin
 from conda.cli.common import specs_from_url, confirm_yn, check_non_admin, ensure_name_or_prefix
-from conda.core.subdir_data import SubdirData
 from conda.history import History
 from conda.core.link import UnlinkLinkTransaction, PrefixSetup
 from conda.cli.install import check_prefix, clone, print_activate
-from conda.base.constants import ChannelPriority, ROOT_ENV_NAME, UpdateModifier, DepsModifier
-from conda.core.solve import diff_for_unlink_link_precs, get_pinned_specs
+from conda.base.constants import ChannelPriority, UpdateModifier, DepsModifier
+from conda.core.solve import get_pinned_specs
 from conda.core.envs_manager import unregister_env
 from conda.core.package_cache_data import PackageCacheData
 from conda.common.compat import on_win
 
 # create support
 from conda.common.path import paths_equal
-from conda.exceptions import (CondaExitZero, CondaImportError, CondaOSError, CondaSystemExit,
+from conda.exceptions import (CondaExitZero, CondaOSError, CondaSystemExit,
                               CondaValueError, DirectoryNotACondaEnvironmentError, CondaEnvironmentError,
-                              DirectoryNotFoundError, DryRunExit, EnvironmentLocationNotFound,
+                              DryRunExit, EnvironmentLocationNotFound,
                               NoBaseEnvironmentError, PackageNotInstalledError, PackagesNotFoundError,
-                              TooManyArgumentsError, UnsatisfiableError)
+                              TooManyArgumentsError)
 
 from conda.gateways.disk.create import mkdir_p
 from conda.gateways.disk.delete import rm_rf, delete_trash, path_is_clean
 from conda.gateways.disk.test import is_conda_environment
 
-from conda._vendor.boltons.setutils import IndexedSet
-from conda.models.prefix_graph import PrefixGraph
-
 from logging import getLogger
 
-import json
-import tempfile
-import logging
-
 import mamba
 import mamba.mamba_api as api
 
-from mamba.utils import get_index, to_package_record_from_subjson, init_api_context
+from mamba.utils import get_index, to_package_record_from_subjson, init_api_context, get_installed_jsonfile, to_txn
+
+if sys.version_info < (3, 2):
+    sys.stdout = codecs.lookup('utf-8')[-1](sys.stdout)
+elif sys.version_info < (3, 7):
+    sys.stdout = codecs.getwriter("utf-8")(sys.stdout.detach())
+else:
+    sys.stdout.reconfigure(encoding='utf-8')
+
 
 log = getLogger(__name__)
 stderrlog = getLogger('conda.stderr')
@@ -88,28 +84,6 @@ class MambaException(Exception):
 
 solver_options = [(api.SOLVER_FLAG_ALLOW_DOWNGRADE, 1)]
 
-def get_installed_packages(prefix, show_channel_urls=None):
-    result = {'packages': {}}
-
-    # Currently, we need to have pip interop disabled :/
-    installed = {rec: rec for rec in PrefixData(prefix, pip_interop_enabled=False).iter_records()}
-
-    # add virtual packages as installed packages
-    # they are packages installed on the system that conda can do nothing about (e.g. glibc)
-    # if another version is needed, installation just fails
-    # they don't exist anywhere (they start with __)
-    _supplement_index_with_system(installed)
-    installed = list(installed)
-
-    for prec in installed:
-        json_rec = prec.dist_fields_dump()
-        json_rec['depends'] = prec.depends
-        json_rec['build'] = prec.build
-        result['packages'][prec.fn] = json_rec
-
-    return installed, result
-
-
 def specs_from_args(args, json=False):
 
     def arg2spec(arg, json=False, update=False):
@@ -130,49 +104,6 @@ def specs_from_args(args, json=False):
 
     return [arg2spec(arg, json=json) for arg in args]
 
-
-def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=[]):
-    to_link_records, to_unlink_records = [], []
-
-    prefix_data = PrefixData(prefix)
-    final_precs = IndexedSet(prefix_data.iter_records())
-
-    lookup_dict = {}
-    for _, c in index:
-        lookup_dict[c.url(with_credentials=True)] = c
-
-    for c, pkg in to_unlink:
-        for i_rec in installed_pkg_recs:
-            if i_rec.fn == pkg:
-                final_precs.remove(i_rec)
-                to_unlink_records.append(i_rec)
-                break
-        else:
-            print("No package record found!")
-
-    for c, pkg, jsn_s in to_link:
-        sdir = lookup_dict[c]
-        rec = to_package_record_from_subjson(sdir, pkg, jsn_s)
-        final_precs.add(rec)
-        to_link_records.append(rec)
-
-    unlink_precs, link_precs = diff_for_unlink_link_precs(prefix,
-                                                          final_precs=IndexedSet(PrefixGraph(final_precs).graph),
-                                                          specs_to_add=specs_to_add,
-                                                          force_reinstall=context.force_reinstall)
-
-    pref_setup = PrefixSetup(
-        target_prefix  = prefix,
-        unlink_precs   = unlink_precs,
-        link_precs     = link_precs,
-        remove_specs   = specs_to_remove,
-        update_specs   = specs_to_add,
-        neutered_specs = ()
-    )
-
-    conda_transaction = UnlinkLinkTransaction(pref_setup)
-    return conda_transaction
-
 use_mamba_experimental = False
 
 def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
@@ -209,17 +140,6 @@ def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
         actions = unlink_link_transaction._make_legacy_action_groups()[0]
         cli_common.stdout_json_success(prefix=prefix, actions=actions)
 
-installed_pkg_recs = None
-
-def get_installed_jsonfile(prefix):
-    global installed_pkg_recs
-    installed_pkg_recs, output = get_installed_packages(prefix, show_channel_urls=True)
-    installed_json_f = tempfile.NamedTemporaryFile('w', delete=False)
-    installed_json_f.write(json_dump(output))
-    installed_json_f.flush()
-    return installed_json_f
-
-
 def remove(args, parser):
     if not (args.all or args.package_names):
         raise CondaValueError('no package names supplied,\n'
@@ -271,7 +191,7 @@ def remove(args, parser):
         if not context.quiet:
             print("Removing specs: {}".format([s.conda_build_form() for s in specs]))
 
-        installed_json_f = get_installed_jsonfile(prefix)
+        installed_json_f, installed_pkg_recs = get_installed_jsonfile(prefix)
 
         mamba_solve_specs = [s.conda_build_form() for s in specs]
 
@@ -311,7 +231,7 @@ def remove(args, parser):
         specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
         specs_to_remove = [MatchSpec(m) for m in mmb_specs[1]]
 
-        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink)
+        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, installed_pkg_recs)
         handle_txn(conda_transaction, prefix, args, False, True)
 
 def install(args, parser, command='install'):
@@ -362,7 +282,7 @@ def install(args, parser, command='install'):
                 # fall-through expected under normal operation
                 pass
         else:
-            if args.mkdir:
+            if hasattr(args, "mkdir") and args.mkdir:
                 try:
                     mkdir_p(prefix)
                 except EnvironmentError as e:
@@ -469,10 +389,7 @@ def install(args, parser, command='install'):
 
         channel_json.append((chan, subdir, priority, subpriority))
 
-    # for c in channel_json:
-    #     print(c[0], c[2], c[3])
-
-    installed_json_f = get_installed_jsonfile(prefix)
+    installed_json_f, installed_pkg_recs = get_installed_jsonfile(prefix)
 
     if isinstall and args.revision:
         get_revision(args.revision, json=context.json)
@@ -485,19 +402,19 @@ def install(args, parser, command='install'):
     # and that they are name-only specs
     if isupdate and context.update_modifier == UpdateModifier.UPDATE_ALL:
         history_dict = History(prefix).get_requested_specs_map()
-        pins = get_pinned_specs(prefix)
-        pin_names = [p.name for p in pins]
-
+        pins = {
+            pin.name: pin
+            for pin in get_pinned_specs(prefix)
+        }
         # for key, match_spec in history_dict.items():
         for key in installed_names:
-
             if key == 'python':
                 i = installed_names.index('python')
                 version = installed_pkg_recs[i].version
                 py_ver = ".".join(version.split(".")[:2]) + '.*'
                 # specs.append(MatchSpec(name="python", version=py_ver))
             else:
-                if key in pin_names:
+                if key in pins:
                     specs.append(pins[key])
                 else:
                     specs.append(MatchSpec(key))
@@ -507,7 +424,7 @@ def install(args, parser, command='install'):
             s = MatchSpec(s)
             if not s.is_name_only_spec:
                 raise CondaValueError("Invalid spec for 'conda update': %s\n"
-                                 "Use 'conda install' instead." % s)
+                                      "Use 'conda install' instead." % s)
             if not prefix_data.get(s.name, None):
                 raise PackageNotInstalledError(prefix, s.name)
 
@@ -620,7 +537,7 @@ def install(args, parser, command='install'):
 
         transaction.execute(prefix_data, PackageCacheData.first_writable().pkgs_dir)
     else:
-        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index)
+        conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, installed_pkg_recs, index)
         handle_txn(conda_transaction, prefix, args, newenv)
 
     try:
@@ -659,6 +576,12 @@ def update(args, parser):
     install(args, parser, 'update')
 
 def repoquery(args, parser):
+
+    if not args.subcmd:
+        print("repoquery needs a subcommand (search, depends or whoneeds)")
+        print("eg:")
+        print("    $ mamba repoquery search xtensor\n")
+        exit(1)
     prefix = context.target_prefix
 
     init_api_context()
@@ -671,7 +594,7 @@ def repoquery(args, parser):
         'use_local': args.use_local
     }
 
-    installed_json_f = get_installed_jsonfile(prefix)
+    installed_json_f, installed_pkg_recs = get_installed_jsonfile(prefix)
 
     pool = api.Pool()
     repos = []
@@ -855,7 +778,7 @@ def main(*args, **kwargs):
 
     args = tuple(ensure_text_type(s) for s in args)
 
-    if len(args) > 2 and args[1] == 'env' and args[2] == 'create':
+    if len(args) > 2 and args[1] == 'env' and args[2] in ('create', 'update'):
         # special handling for conda env create!
         from mamba import mamba_env
         return mamba_env.main()
